
openGraphics <-
    function(..., mode)
{
    ## event handler to save an image
    saveimage.menu <- function(action, win) {
        dialog <- gtkFileChooserDialog("Enter a file name", win, "save",
                                       "gtk-cancel", GtkResponseType["cancel"],
                                       "gtk-save", GtkResponseType["accept"])
        if (dialog$run() == GtkResponseType["accept"]) {
            filename <- dialog$getFilename()
            dev.copy(png, filename)
            dev.off()
        }
        dialog$destroy()
    }
    
    ## event handler to print an image
    print.menu <- function(action, win){
      show.message("fixing errors....")
      ##  printing()
    }

    ## event handler to close the plot
    quit.menu <- function(action, win){
        win$destroy()
    }

    ## setup the plot dialog window
    entries <- list(
                    ## name, stock.id (prefab icons), label (. signifies mneumonic)
                   list("SaveMenu", NULL, "_Save"),
                   list("PrintMenu", NULL, "_Print"),
                   list("QuitMenu", NULL, "_Quit"),
                    list("SaveAs", "gtk-save", "SaveAsImage...", NULL, "Save to a file",
                         saveimage.menu),
                    list("Print", "gtk-print", "_Print", "<control>P", "Print the current graph",
                         print.menu), 
                    list("Quit", "gtk-quit", "_Quit", "<control>Q", "Quit", quit.menu))
    
    win <- gtkWindow(show = FALSE)
    win$setTitle("Graphics")
    
    ## add a table layout
    table <- gtkTableNew(2, 4, FALSE)
    win$add(table)
    agroup <- gtkActionGroupNew("graphicsAtions")

    ## add actions to group with the window widget passed to callbacks
    agroup$addActions(entries, win)

    ## create a UI manager to read in menus specified in XML
    manager <- gtkUIManagerNew()

    win$setData("ui-manager", manager)
    manager$insertActionGroup(agroup, 0)

    win$addAccelGroup(manager$getAccelGroup())
    
    ## Define some XML
    uistr <- paste(
                   "<ui>",
                   "  <toolbar  name='ToolBar'>",
                   "    <toolitem action='SaveAs'/>",
                   "    <toolitem action='Print'/>",
                   "    <toolitem action='Quit'/>", 
                   "    <separator action='Sep1'/>",
                   "  </toolbar>",
                   "</ui>", sep="\n")

    manager$addUiFromString(uistr)

    bar <- manager$getWidget("/ToolBar")
    bar$setTooltips(TRUE)
    bar$show()
    table$attach(bar, 0, 1, 1, 2, c("expand", "fill"), 0, 0, 0)
    
    ##### for drawing plots ######
    HPaned <- gtkHPanedNew()
    HPaned$setBorderWidth(5)
    
    graphics<- gtkDrawingArea()
    dev_list <- list()
    dev_list[[1]] <- graphics
    
    sw <- make.scrolled_window(dev_list[[1]])
    
    hbox <- gtkHBox(TRUE, 0)
    hbox$packStart(sw, TRUE, TRUE, 0) # for data

    HPaned$add(hbox)
    
    sz <- 600
    
    ## draw multiple plots 
    if(mode == 2){
            n <- length(iget("value.x"))
            if(n != 1){
           	 for(j in 2:length(iget("value.x"))){
           	        graphics<- gtkDrawingArea()
           	 	dev_list[[j]] <- graphics
           	 	
           	 	sw <- make.scrolled_window(dev_list[[j]])
           	 	hbox$packStart(sw, TRUE, TRUE, 0)
           	 	asCairoDevice(dev_list[[j]])
           	 	}
           	 sz <- sz+200
            }
    }
    
    table$attach(HPaned, 0, 1, 2, 3, c("expand", "fill"), c("expand", "fill"), 0, 0)
    win$setDefaultSize(sz, 600)
    win$showAll()

    ## embedd the output in a Cairo device
    asCairoDevice(dev_list[[1]])

    if(mode == 1) ## regular plots
    {
        tryCatch(graph.selection(...),
                 error = function(e) {
                     m <- paste("Error:", conditionMessage(e))
                     error.handler(m)
                 }, 
                 warning = function(e) {
                     m <- paste("Warning:", conditionMessage(e))
                     warning.handler(m)
                 })
    }

    ######### graphics for backgating ######################################
    else if (mode == 2){
        tryCatch(
             { 
                 dat <- activeFlowset()
                 gating <- .guiEnv$Gate.list
                 inds <- iget("value.x")
            
                 for(i in 1:length(inds))
                 {
                     if(i == 1) title <- paste(activeNodeName(),names(gating[inds[1]]), sep="_" )
                     else title <- paste(title, names(gating[inds[i]]), sep="_")
                    
                     filter.result <- filter(dat, gating[[inds[i]]])
                   
                     if(length(gating[[inds[i]]]@parameters) == 2) {
                               	        
                          if(gating[[inds[i]]]@filterId == "defaultQuadGate"){
                     	              x<-gating[[inds[i]]]@boundary[1]
                     	              y<-gating[[inds[i]]]@boundary[2] }
                       	  else if(gating[[inds[i]]]@filterId == "defaultRectangleGate") {             
                     	              x<- names(gating[[inds[i]]]@min)[1]
                                      y <-names(gating[[inds[i]]]@min)[2] }  
                          else{
                                      x <- gating[[inds[i]]]@parameters[[1]]@parameters
                     	              y <- gating[[inds[i]]]@parameters[[2]]@parameters  }                          
                               
                     	
                     	  flowViz.par.set(gate=list(fill="red", alpha=0.3), gate.text=list(col="black", alpha=0.9, cex=0.6))
                     	  form <- sprintf("`%s` ~ `%s`",y ,x)
                          fcall <- sprintf("xyplot(%s, data=dat, filter=filter.result, smooth=TRUE, outline=TRUE, names=TRUE, main = title)", form)
                          plot(eval(parse(text = fcall)))  }
                      else{
                     	  x <- gating[[inds[i]]]@parameters[[1]]@parameters 
                     	  form <- sprintf("~ `%s`", x)
                          fcall <- sprintf("densityplot(%s, data=dat, filter=filter.result )", form)
                          plot(eval(parse(text = fcall))) 
                        }
                                 
                     if(length(inds) > 1){
                          small <- summary(filter.result)  
                          sub_dat <- Subset(dat, gating[[inds[i]]])
                          make.contents(sub_dat,small)
                          dat <- sub_dat
                     }
                     
                     dev.set(dev.next())
                                      
                 }
              
              },
                 error = function(e) {
                     m <- paste("Error:", conditionMessage(e))
                     error.handler(m)
                 }, 
                 warning = function(e) {
                     m <- paste("Warning:", conditionMessage(e))
                     warning.handler(m)
                 }
                 
                 )
        pop <- 1
        iset("pop", pop)
        
    }

    ## for rectangle gate...
    else if(mode == 3){
          tryCatch(
          { 
            x <- activeFlowset()
            
            form <- sprintf("`%s` ~ `%s`", value.y, value.x) 
          #  fcall <- sprintf("plot(xyplot(%s, data=dt))", form)
          #  evaluateCommand(fcall)
             fcall <- sprintf("xyplot(%s, data=dt)", form)
          plot(eval(parse(text = fcall))) }, 
          
           error = function(e) {
                m <- paste("Error was produced.", conditionMessage(e))
                error.handler(m)
            }, 
            warning = function(e) {
               m <- paste("Waring was produced.", conditionMessage(e))
               warning.handler(m)
            }
    
          )   
             pop <- 1
            iset("pop", pop)
           
    }
    
    ## for hierarchy of workflow object...
    else if(mode == 4){
        tryCatch(
        { plot(activeWorkflow()) }, 
         error = function(e) {
                m <- paste("Error was produced.", conditionMessage(e))
                error.handler(m)
            }, 
            warning = function(e) {
               m <- paste("Waring was produced.", conditionMessage(e))
               warning.handler(m)
            }
          )  
    }
    ## for probin
    else{
               value.x <- c(iget("value.x"), iget("value.y"))
               resCtrl <- iget("variable.y")
               stat <- iget("variable.x")
               fset <- iget("dfrm")
               if(length(fset) == 1)
                   plotBins(resCtrl,fset[[1]],channels=value.x,title="Binned control data")
               else{
                   title <- paste("Comparison", identifier(fset[[1]]), "&", identifier(fset[[2]]))
                   plotBins(resCtrl,data=fset[[2]],channels=value.x,title= title,
                            residuals=stat$residuals[2,],shadeFactor=0.7) 
      
       }
    }
    
}




## call a lattice densityplot 
densPlot <- function(channels){
    channels <- paste(sprintf("`%s`", channels), collapse=" + ", sep="")
    fcall <- sprintf("plot(densityplot(~ %s, data=dt))", channels)
    print("Creating densityplot...")
    evaluateCommand(fcall)  
    pop <- 1
    iset("pop", pop)
  
}

## call a lattice xyplot
xyPlot <- function(x, y){
    fcall <- sprintf("plot(xyplot(`%s`~`%s`, data=dt))", y, x)
    print("Creating xyplot...")
    evaluateCommand(fcall) 
    pop <- 1
    iset("pop", pop)
  
}
